@@ -31,6 +31,7 @@ Supported `SmartConsumer` MIME types are:
 import bz2
 import contextlib
 import csv
+import datetime
 import io
 import json
 import logging
@@ -47,7 +48,7 @@ from pika import exceptions
 from tornado import concurrent, gen, locks
 import yaml
 
-from rejected import data, log
+from . import data, log
 
 LOGGER = logging.getLogger(__name__)
 
@@ -72,6 +73,7 @@ except NameError:
 
 
 DEFAULT_CHANNEL = 'default'
+_DROPPED_MESSAGE = 'X-Rejected-Dropped'
 _PROCESSING_EXCEPTIONS = 'X-Processing-Exceptions'
 _EXCEPTION_FROM = 'X-Exception-From'
 
@@ -95,15 +97,35 @@ class Consumer(object):
     (or if the ``DROP_INVALID_MESSAGES`` attribute is set to ``True``).
     If it is ``False``, a :exc:`~rejected.consumer.MessageException` is raised.
 
+    If ``DROP_EXCHANGE`` is specified either as an attribute of the consumer
+    class or in the consumer configuration, if a message is dropped, it is
+    published to the that exchange prior to rejecting the message in RabbitMQ.
+    When the message is republished, four new values are added to the AMQP
+    ``headers`` message property: ``X-Dropped-By``, ``X-Dropped-Reason``,
+    ``X-Dropped-Timestamp``, ``X-Original-Exchange``.
+
+    The ``X-Dropped-By`` header value contains the configured name of the
+    consumer that dropped the message. ``X-Dropped-Reason`` contains the
+    reason the message was dropped (eg invalid message type or maximum error
+    count). ``X-Dropped-Timestamp`` value contains the ISO-8601 formatted
+    timestamp of when the message was dropped. Finally, the
+    ``X-Original-Exchange`` value contains the original exchange that the
+    message was published to.
+
     If a consumer raises a :exc:`~rejected.consumer.ProcessingException`, the
     message that was being processed will be republished to the exchange
     specified by the ``error`` exchange configuration value or the
     ``ERROR_EXCHANGE`` attribute of the consumer's class. The message will be
     published using the routing key that was last used for the message. The
-    original message body and properties will be used and an additional header
-    ``X-Processing-Exceptions`` will be added that will contain the number of
-    times the message has had a :exc:`~rejected.consumer.ProcessingException`
-    raised for it. In combination with a queue that has ``x-message-ttl`` set
+    original message body and properties will be used and two additional
+    header property values may be added:
+
+        - ``X-Processing-Exception`` contains the string value of the
+            exception that was raised, if specified.
+        - ``X-Processing-Exceptions`` contains the quantity of processing
+            exceptions that have been raised for the message.
+
+    In combination with a queue that has ``x-message-ttl`` set
     and ``x-dead-letter-exchange`` that points to the original exchange for the
     queue the consumer is consuming off of, you can implement a delayed retry
     cycle for messages that are failing to process due to external resource or
@@ -115,21 +137,44 @@ class Consumer(object):
     greater than or equal to the specified value, the message will
     be dropped.
 
+    As of 3.18.6, the ``MESSAGE_AGE_KEY`` class level attribute contains the
+    default key part to used when recording stats for the message age. You can
+    also override the :py:meth:`~rejected.consumer.Consumer.message_age_key`
+    method to create compound keys. For example, to create a key that includes
+    the message priority:
+
+    .. code:: python
+
+        class Consumer(consumer.Consumer):
+
+            def message_age_key(self):
+                return 'priority-{}.message_age'.format(self.priority or 0)
+
     .. note:: Since 3.17, :class:`~rejected.consumer.Consumer` and
         :class:`~rejected.consumer.PublishingConsumer` have been combined
         into the same class.
 
+    As of 3.19.13, the ``ACK_PROCESSING_EXCEPTIONS`` class level attribute
+    allows you to ack messages that raise a
+    :exc:`~rejected.consumer.ProcessingException` instead of rejecting them,
+    allowing for dead-lettered messages to be constrained to
+    :exc:`~rejected.consumer.MessageException`s only. Defaults to `False`.
+
     """
+    DROP_EXCHANGE = None
     DROP_INVALID_MESSAGES = False
     MESSAGE_TYPE = None
     ERROR_EXCHANGE = 'errors'
     ERROR_MAX_RETRY = None
+    MESSAGE_AGE_KEY = 'message_age'
+    ACK_PROCESSING_EXCEPTIONS = False
 
     def __init__(self, settings, process,
                  drop_invalid_messages=None,
                  message_type=None,
                  error_exchange=None,
-                 error_max_retry=None):
+                 error_max_retry=None,
+                 drop_exchange=None):
         """Creates a new instance of the :class:`~rejected.consumer.Consumer`
         class. To perform initialization tasks, extend
         :meth:`~rejected.consumer.Consumer.initialize`.
@@ -137,6 +182,7 @@ class Consumer(object):
         """
         self._channels = {}
         self._correlation_id = None
+        self._drop_exchange = drop_exchange or self.DROP_EXCHANGE
         self._drop_invalid = drop_invalid_messages or \
             self.DROP_INVALID_MESSAGES
         self._error_exchange = error_exchange or self.ERROR_EXCHANGE
@@ -198,6 +244,18 @@ class Consumer(object):
         """
         raise NotImplementedError
 
+    def message_age_key(self):
+        """Return the key part that is used in submitting message age stats.
+        Override this method to change the key part. This could be used to
+        include message priority in the key, for example.
+
+        .. versionadded:: 3.18.6
+
+        :rtype: str
+
+        """
+        return self.MESSAGE_AGE_KEY
+
     def on_finish(self):
         """Called after a message has been processed.
 
@@ -277,7 +335,8 @@ class Consumer(object):
             specified, the channel that the message was delivered on is used.
 
         """
-        self.logger.debug('Publishing message to %s:%s', exchange, routing_key)
+        self.logger.debug('Publishing message to %s:%s (%s)',
+                          exchange, routing_key, channel)
         with self._measurement.track_duration(
                 'publish.{}.{}'.format(exchange, routing_key)):
             self._publish_channel(channel).basic_publish(
@@ -357,35 +416,50 @@ class Consumer(object):
 
         """
         if self.sentry_client:
-            self.logger.debug('Setting sentry context for %s to %s', tag, value)
+            self.logger.debug(
+                'Setting sentry context for %s to %s', tag, value)
             self.sentry_client.tags_context({tag: value})
 
-    def stats_add_timing(self, key, duration):
-        """Add a timing to the per-message measurements
+    def stats_add_duration(self, key, duration):
+        """Add a duration to the per-message measurements
 
-        .. versionadded:: 3.13.0
+        .. versionadded:: 3.19.0
 
         :param str key: The key to add the timing to
-        :param int|float duration: The timing value
+        :param int|float duration: The timing value in seconds
 
         """
         if not self._measurement:
             LOGGER.warning('stats_add_timing invoked outside execution')
             return
-        self._measurement.set_value(key, duration)
+        self._measurement.add_duration(key, duration)
+
+    def stats_add_timing(self, key, duration):
+        """Add a timing to the per-message measurements
+
+        .. versionadded:: 3.13.0
+        .. deprecated:: 3.19.0
+
+        :param str key: The key to add the timing to
+        :param int|float duration: The timing value in seconds
+
+        """
+        warnings.warn('Deprecated, use Consumer.stats_add_duration',
+                      DeprecationWarning)
+        self.stats_add_duration(key, duration)
 
     def statsd_add_timing(self, key, duration):
         """Add a timing to the per-message measurements
 
         :param str key: The key to add the timing to
-        :param int|float duration: The timing value
+        :param int|float duration: The timing value in seconds
 
         .. deprecated:: 3.13.0
 
         """
-        warnings.warn('Deprecated, use Consumer.stats_add_timing',
+        warnings.warn('Deprecated, use Consumer.stats_add_duration',
                       DeprecationWarning)
-        self.stats_add_timing(key, duration)
+        self.stats_add_duration(key, duration)
 
     def stats_incr(self, key, value=1):
         """Increment the specified key in the per-message measurements
@@ -447,6 +521,7 @@ class Consumer(object):
         """Time around a context and add to the the per-message measurements
 
         .. versionadded:: 3.13.0
+        .. deprecated:: 3.19.0
 
         :param str key: The key for the timing to track
 
@@ -455,7 +530,7 @@ class Consumer(object):
         try:
             yield
         finally:
-            self.stats_add_timing(
+            self.stats_add_duration(
                 key, max(start_time, time.time()) - start_time)
 
     def statsd_track_duration(self, key):
@@ -487,7 +562,7 @@ class Consumer(object):
         """
         try:
             yield self._yield_condition.wait(
-                self._channel.connection.ioloop.time() + 0.001)
+                self._message.channel.connection.ioloop.time() + 0.001)
         except gen.TimeoutError:
             pass
 
@@ -587,7 +662,19 @@ class Consumer(object):
         """
         if not self._message:
             return None
-        return self._message.properties.headers or dict()
+        return self._message.properties.headers or {}
+
+    @property
+    def io_loop(self):
+        """Access the :py:class:`tornado.ioloop.IOLoop` instance for the
+        current message.
+
+        .. versionadded:: 3.18.4
+
+        :rtype: tornado.ioloop.IOLoop
+
+        """
+        return self._message.channel.connection.ioloop
 
     @property
     def message_id(self):
@@ -764,6 +851,14 @@ class Consumer(object):
         self._message = message_in
         self._measurement = measurement
 
+        # If timestamp is set, record age of the message coming in
+        if message_in.properties.timestamp:
+            message_age = float(
+                max(message_in.properties.timestamp, time.time()) -
+                message_in.properties.timestamp)
+            if message_age > 0:
+                measurement.add_duration(self.message_age_key(), message_age)
+
         # Ensure there is a correlation ID
         self._correlation_id = message_in.properties.correlation_id or \
             message_in.properties.message_id or \
@@ -784,6 +879,8 @@ class Consumer(object):
                                     self.message_type)
                 # Should the message be dropped or returned to the broker?
                 if self._drop_invalid:
+                    if self._drop_exchange:
+                        self._republish_dropped_message('invalid type')
                     raise gen.Return(data.MESSAGE_DROP)
                 raise gen.Return(data.MESSAGE_EXCEPTION)
 
@@ -794,6 +891,10 @@ class Consumer(object):
                 self.logger.warning('Dropping message with %i deaths due to '
                                     'ERROR_MAX_RETRY',
                                     self.headers[_PROCESSING_EXCEPTIONS])
+                if self._drop_exchange:
+                    self._republish_dropped_message(
+                        'max retries ({})'.format(
+                            self.headers[_PROCESSING_EXCEPTIONS]))
                 raise gen.Return(data.MESSAGE_DROP)
 
         result = None
@@ -820,38 +921,55 @@ class Consumer(object):
 
         except exceptions.ConnectionClosed as error:
             self.logger.critical('Connection closed while processing %s: %s',
-                                 message_in.delivery_tag, error)
+                                 message_in.delivery_tag, str(error))
             self._measurement.set_tag('exception', error.__class__.__name__)
             raise gen.Return(None)
 
         except ConsumerException as error:
             self.logger.error('ConsumerException processing delivery %s: %s',
-                              message_in.delivery_tag, error)
+                              message_in.delivery_tag, str(error))
             self._measurement.set_tag('exception', error.__class__.__name__)
+            if error.metric:
+                self._measurement.set_tag('error', error.metric)
             raise gen.Return(data.CONSUMER_EXCEPTION)
 
         except MessageException as error:
-            self.logger.debug('MessageException processing delivery %s: %s',
-                              message_in.delivery_tag, error)
+            self.logger.info('MessageException processing delivery %s: %s',
+                             message_in.delivery_tag, str(error))
             self._measurement.set_tag('exception', error.__class__.__name__)
+            if error.metric:
+                self._measurement.set_tag('error', error.metric)
             raise gen.Return(data.MESSAGE_EXCEPTION)
 
         except ProcessingException as error:
-            self.logger.debug('ProcessingException processing delivery %s: %s',
-                              message_in.delivery_tag, error)
+            self.logger.warning(
+                'ProcessingException processing delivery %s: %s',
+                message_in.delivery_tag, str(error))
             self._measurement.set_tag('exception', error.__class__.__name__)
-            self._republish_processing_error()
+            if error.metric:
+                self._measurement.set_tag('error', error.metric)
+            self._republish_processing_error(
+                error.metric or error.__class__.__name__)
             raise gen.Return(data.PROCESSING_EXCEPTION)
 
+        except NotImplementedError as error:
+            self.log_exception('NotImplementedError processing delivery'
+                               ' %s: %s', message_in.delivery_tag, error)
+            self._measurement.set_tag('exception', 'UnhandledException')
+            raise gen.Return(data.UNHANDLED_EXCEPTION)
+
         except Exception as error:
             exc_info = sys.exc_info()
             if concurrent.is_future(result):
                 error = result.exception()
-                exc_info = result.exc_info()
+                try:
+                    exc_info = result.exc_info()
+                except AttributeError:
+                    pass
             self.log_exception('Exception processing delivery %s: %s',
-                               message_in.delivery_tag, error,
+                               message_in.delivery_tag, str(error),
                                exc_info=exc_info)
-            self._measurement.set_tag('exception', error.__class__.__name__)
+            self._measurement.set_tag('exception', 'UnhandledException')
             raise gen.Return(data.UNHANDLED_EXCEPTION)
 
         if not self._finished:
@@ -908,9 +1026,10 @@ class Consumer(object):
         :param str feature: A friendly name for the setting feature
 
         """
-        if not self.settings.get(name):
-            raise Exception("You must define the '%s' setting in your "
-                            "application to use %s" % (name, feature))
+        if name not in self.settings:
+            raise Exception(
+                'You must define the "{}" setting in to use {}'.format(
+                    name, feature))
 
     def set_channel(self, name, channel):
         """Assign the _channel attribute to the channel that was passed in.
@@ -956,13 +1075,47 @@ class Consumer(object):
                 setattr(properties, key, properties_in.get(key))
         return properties
 
-    def _publish_channel(self, name):
+    def _publish_channel(self, name=None):
+        """Return the channel to publish onm optionally specifying the channel
+        name to use.
+
+        :param str name:
+        :rtype: pika.channel.Channel
+
+        """
+        if not name:
+            return self._message.channel
         try:
-            return self._message.channel if not name else self._channels[name]
+            return self._channels[name]
         except KeyError:
             raise ValueError('Channel {} not found'.format(name))
 
-    def _republish_processing_error(self):
+    def _republish_dropped_message(self, reason):
+        """Republish the original message that was received it is being dropped
+        by the consumer.
+
+        This for internal use and should not be extended or used directly.
+
+        :param str reason: The reason the message was dropped
+
+        """
+        self.logger.debug('Republishing due to ProcessingException')
+        properties = dict(self._message.properties) or {}
+        if 'headers' not in properties or not properties['headers']:
+            properties['headers'] = {}
+        properties['headers']['X-Dropped-By'] = self.name
+        properties['headers']['X-Dropped-Reason'] = reason
+        properties['headers']['X-Dropped-Timestamp'] = \
+            datetime.datetime.utcnow().isoformat()
+        properties['headers']['X-Original-Exchange'] = self._message.exchange
+
+        self._message.channel.basic_publish(
+            self._drop_exchange,
+            self._message.routing_key,
+            self._message.body,
+            pika.BasicProperties(**properties))
+
+    def _republish_processing_error(self, error):
         """Republish the original message that was received because a
         :exc:`~rejected.consumer.ProcessingException` was raised.
 
@@ -971,12 +1124,17 @@ class Consumer(object):
         Add a header that keeps track of how many times this has happened
         for this message.
 
+        :param str error: The string value for the exception
+
         """
         self.logger.debug('Republishing due to ProcessingException')
         properties = dict(self._message.properties) or {}
         if 'headers' not in properties or not properties['headers']:
             properties['headers'] = {}
 
+        if error:
+            properties['headers']['X-Processing-Exception'] = error
+
         if _PROCESSING_EXCEPTIONS not in properties['headers']:
             properties['headers'][_PROCESSING_EXCEPTIONS] = 1
         else:
@@ -1255,7 +1413,7 @@ class SmartConsumer(Consumer):
     def _dump_json_value(value):
         """Serialize a value into JSON
 
-        :param str|dict|list: The value to serialize as JSON
+        :param object value: The value to serialize
         :rtype: bytes
 
         """
@@ -1265,7 +1423,8 @@ class SmartConsumer(Consumer):
     def _dump_msgpack_value(value):
         """Serialize a value into MessagePack
 
-        :param str|dict|list: The value to serialize as msgpack
+        :param object value: The value to serialize
+        :type value: str or dict or list
         :rtype: bytes
 
         """
@@ -1275,7 +1434,7 @@ class SmartConsumer(Consumer):
     def _dump_pickle_value(value):
         """Serialize a value into the pickle format
 
-        :param any value: The object to pickle
+        :param object value: The object to pickle
         :rtype: bytes
 
         """
@@ -1298,9 +1457,9 @@ class SmartConsumer(Consumer):
 
     @staticmethod
     def _dump_yaml_value(value):
-        """Dump a dict into a YAML string
+        """Dump an object into a YAML string
 
-        :param dict value: The value to dump as a YAML string
+        :param object value: The value to dump as a YAML string
         :rtype: str
 
         """
@@ -1372,7 +1531,7 @@ class SmartConsumer(Consumer):
         if isinstance(value, bytes):
             value = value.decode('utf-8')
         try:
-            return json.loads(value, encoding='utf-8')
+            return json.loads(value)
         except ValueError as error:
             self.logger.exception('Could not decode message body: %s', error)
             raise MessageException(error)
@@ -1444,27 +1603,78 @@ class SmartPublishingConsumer(SmartConsumer):
         super(SmartPublishingConsumer, self).__init__(*args, **kwargs)
 
 
-class ConsumerException(Exception):
+class RejectedException(Exception):
+    """Base exception for :py:class:`~rejected.consumer.Consumer` related
+    exceptions.
+
+    If provided, the metric will be used to automatically record exception
+    metric counts using the path
+    `[prefix].[consumer-name].exceptions.[exception-type].[metric]`.
+
+    Positional and keyword arguments are used to format the value that is
+    passed in when providing the string value of the exception.
+
+    :param str value: An optional value used in string representation
+    :param str metric: An optional value for auto-instrumentation of exceptions
+
+    .. versionadded:: 3.19.0
+
+    """
+    METRIC_NAME = 'rejected-exception'
+
+    def __init__(self, *args, **kwargs):
+        if len(args) > 1:
+            self.args = args[1:] if 'value' not in kwargs else args
+        else:
+            self.args = args
+        self.metric = kwargs.pop('metric', None)
+        self.value = kwargs.pop('value', '{!r} {!r}' if not args else args[0])
+        self.kwargs = kwargs
+
+    def __str__(self):
+        if not self.args and not self.kwargs:
+            return repr(self)
+        return self.value.format(*self.args, **self.kwargs)
+
+    def __repr__(self):
+        if not self.args and not self.kwargs:
+            return '{}()'.format(self.__class__.__name__)
+        return '{}({})'.format(self.__class__.__name__, str(self))
+
+
+class ConsumerException(RejectedException):
     """May be called when processing a message to indicate a problem that the
     Consumer may be experiencing that should cause it to stop.
 
+    :param str value: An optional value used in string representation
+    :param str metric: An optional value for auto-instrumentation of exceptions
+
     """
-    pass
+    def __init__(self, *args, **kwargs):
+        super(ConsumerException, self).__init__(*args, **kwargs)
 
 
-class MessageException(Exception):
-    """Invoke when a message should be rejected and not requeued, but not due
+class MessageException(RejectedException):
+    """Invoke when a message should be rejected and not re-queued, but not due
     to a processing error that should cause the consumer to stop.
 
+    :param str value: An optional value used in string representation
+    :param str metric: An optional value for auto-instrumentation of exceptions
+
     """
-    pass
+    def __init__(self, *args, **kwargs):
+        super(MessageException, self).__init__(*args, **kwargs)
 
 
-class ProcessingException(Exception):
-    """Invoke when a message should be rejected and not requeued, but not due
+class ProcessingException(RejectedException):
+    """Invoke when a message should be rejected and not re-queued, but not due
     to a processing error that should cause the consumer to stop. This should
     be used for when you want to reject a message which will be republished to
     a retry queue, without anything being stated about the exception.
 
+    :param str value: An optional value used in string representation
+    :param str metric: An optional value for auto-instrumentation of exceptions
+
     """
-    pass
+    def __init__(self, *args, **kwargs):
+        super(ProcessingException, self).__init__(*args, **kwargs)
