@@ -6,7 +6,6 @@ import collections
 import logging
 import multiprocessing
 import os
-import psutil
 try:
     import Queue as queue
 except ImportError:
@@ -15,10 +14,19 @@ import signal
 import sys
 import time
 
-from rejected import state, process, __version__
+import psutil
+
+from . import __version__, process, state
 
 LOGGER = logging.getLogger(__name__)
 
+_PROCESS_RUNNING = [psutil.STATUS_RUNNING, psutil.STATUS_SLEEPING]
+_PROCESS_STOPPED_OR_DEAD = [psutil.STATUS_STOPPED, psutil.STATUS_DEAD]
+
+if sys.version_info < (3, 0, 0):
+    FileNotFoundError = psutil.NoSuchProcess
+    ProcessLookupError = OSError
+
 
 class Consumer(object):
     """Class used for keeping track of each consumer type being managed by
@@ -60,14 +68,14 @@ class MasterControlProgram(state.State):
         # Default values
         self._active_cache = None
         self.consumer_cfg = self.get_consumer_cfg(config, consumer, quantity)
-        self.consumers = dict()
+        self.consumers = {}
         self.config = config
-        self.last_poll_results = dict()
+        self.last_poll_results = {}
         self.poll_data = {'time': 0, 'processes': []}
         self.poll_timer = None
         self.profile = profile
         self.results_timer = None
-        self.stats = dict()
+        self.stats = {}
         self.stats_queue = multiprocessing.Queue()
         self.polled = False
         self.unresponsive = collections.Counter()
@@ -91,25 +99,31 @@ class MasterControlProgram(state.State):
         :rtype: list
 
         """
+        LOGGER.debug('Checking active processes (cache: %s)', use_cache)
         if use_cache and self._active_cache and \
                 self._active_cache[0] > time.time() - self.poll_interval:
             return self._active_cache[1]
-        active_processes, dead_processes = list(), list()
+        active_processes, dead_processes = [], []
         for consumer in self.consumers:
-            for name in self.consumers[consumer].processes:
+            for name in list(self.consumers[consumer].processes.keys()):
                 child = self.get_consumer_process(consumer, name)
-                if int(child.pid) == os.getpid():
+                if child is None:
+                    continue
+                if child.pid is None:
+                    dead_processes.append((consumer, name))
+                    continue
+                elif int(child.pid) == os.getpid():
                     continue
                 try:
                     proc = psutil.Process(child.pid)
-                except psutil.NoSuchProcess:
+                except (FileNotFoundError, psutil.NoSuchProcess):
                     dead_processes.append((consumer, name))
                     continue
 
                 if self.unresponsive[name] >= self.MAX_UNRESPONSIVE_COUNT:
-                    LOGGER.info('Killing unresponsive consumer %s (%i): '
-                                '%i misses',
-                                name, proc.pid, self.unresponsive[name])
+                    LOGGER.info(
+                        'Killing unresponsive consumer %s (%i): '
+                        '%i misses', name, proc.pid, self.unresponsive[name])
                     try:
                         os.kill(child.pid, signal.SIGABRT)
                     except OSError:
@@ -139,7 +153,7 @@ class MasterControlProgram(state.State):
 
         # Iterate through the last poll results
         stats = self.consumer_stats_counter()
-        consumer_stats = dict()
+        consumer_stats = {}
         for name in data.keys():
             consumer_stats[name] = self.consumer_stats_counter()
             consumer_stats[name]['processes'] = self.process_count(name)
@@ -188,7 +202,7 @@ class MasterControlProgram(state.State):
 
         # Add it to our last poll global data
         if consumer_name not in self.last_poll_results:
-            self.last_poll_results[consumer_name] = dict()
+            self.last_poll_results[consumer_name] = {}
         self.last_poll_results[consumer_name][process_name] = data_values
 
         # Calculate the stats
@@ -256,24 +270,29 @@ class MasterControlProgram(state.State):
         :rtype: bool
 
         """
-        status = proc.status()
+        LOGGER.debug('Checking %s (%r)', name, proc)
+        try:
+            status = proc.status()
+        except psutil.NoSuchProcess:
+            LOGGER.debug('NoSuchProcess: %s (%r)', name, proc)
+            return True
+
         LOGGER.debug('Process %s (%s) status: %r (Unresponsive Count: %s)',
                      name, proc.pid, status, self.unresponsive[name])
-        try:
-            if status in [psutil.STATUS_RUNNING,
-                          psutil.STATUS_SLEEPING]:
-                return False
-            if status == psutil.STATUS_ZOMBIE:
+        if status in _PROCESS_RUNNING:
+            return False
+        elif status == psutil.STATUS_ZOMBIE:
+            try:
+                proc.wait(0.1)
+            except psutil.TimeoutExpired:
+                pass
+            try:
                 proc.terminate()
+                status = proc.status()
+            except psutil.NoSuchProcess:
+                LOGGER.debug('NoSuchProcess: %s (%r)', name, proc)
                 return True
-            elif status == psutil.STATUS_STOPPED:
-                return True
-            elif status == psutil.STATUS_DEAD:
-                return True
-        except psutil.NoSuchProcess:
-            LOGGER.debug('Process is dead and does not exist')
-            return True
-        return False
+        return status in _PROCESS_STOPPED_OR_DEAD
 
     def kill_processes(self):
         """Gets called on shutdown by the timer when too much time has gone by,
@@ -324,17 +343,16 @@ class MasterControlProgram(state.State):
                         self.stats['consumers'][key]['processed'],
                         self.stats['consumers'][key]['failed'])
 
-    def new_consumer(self, config):
+    def new_consumer(self, config, consumer_name):
         """Return a consumer dict for the given name and configuration.
 
         :param dict config: The consumer configuration
+        :param str consumer_name: The consumer name
         :rtype: dict
 
         """
-        return Consumer(0,
-                        dict(),
-                        config.get('qty', self.DEFAULT_CONSUMER_QTY),
-                        config['queue'])
+        return Consumer(0, {}, config.get('qty', self.DEFAULT_CONSUMER_QTY),
+                        config.get('queue', consumer_name))
 
     def new_process(self, consumer_name):
         """Create a new consumer instances
@@ -366,14 +384,26 @@ class MasterControlProgram(state.State):
         self.consumers[name].last_proc_num += 1
         return self.consumers[name].last_proc_num
 
-    def on_abort(self, _signum, _unused_frame):
-        LOGGER.debug('Abort signal received from child')
-        time.sleep(1)
-        if not self.active_processes():
+    def on_sigchld(self, _signum, _unused_frame):
+        """Invoked when a child sends up an SIGCHLD signal.
+
+        :param int _signum: The signal that was invoked
+        :param frame _unused_frame: The frame that was interrupted
+
+        """
+        LOGGER.info('SIGCHLD received from child')
+        if not self.active_processes(False):
             LOGGER.info('Stopping with no active processes and child error')
+            signal.setitimer(signal.ITIMER_REAL, 0, 0)
             self.set_state(self.STATE_STOPPED)
 
     def on_timer(self, _signum, _unused_frame):
+        """Invoked by the Poll timer signal.
+
+        :param int _signum: The signal that was invoked
+        :param frame _unused_frame: The frame that was interrupted
+
+        """
         if self.is_shutting_down:
             LOGGER.debug('Polling timer fired while shutting down')
             return
@@ -413,7 +443,7 @@ class MasterControlProgram(state.State):
             return self.check_process_counts()
 
         # Start our data collection dict
-        self.poll_data = {'timestamp': time.time(), 'processes': list()}
+        self.poll_data = {'timestamp': time.time(), 'processes': []}
 
         # Iterate through all of the consumers
         for proc in list(self.active_processes()):
@@ -421,8 +451,13 @@ class MasterControlProgram(state.State):
                 continue
 
             # Send the profile signal
-            os.kill(int(proc.pid), signal.SIGPROF)
-            self.poll_data['processes'].append(proc.name)
+            try:
+                os.kill(int(proc.pid), signal.SIGPROF)
+            except ProcessLookupError as error:
+                LOGGER.warning('Error sending SIGPROF to %s: %s',
+                               proc.pid, error)
+            else:
+                self.poll_data['processes'].append(proc.name)
 
         # Check if we need to start more processes
         self.check_process_counts()
@@ -433,7 +468,8 @@ class MasterControlProgram(state.State):
         :rtype: bool
 
         """
-        return (time.time() - self.poll_data['timestamp']) >= self.poll_interval
+        return ((time.time() - self.poll_data['timestamp']) >=
+                self.poll_interval)
 
     def poll_results_check(self):
         """Check the polling results by checking to see if the stats queue is
@@ -494,17 +530,28 @@ class MasterControlProgram(state.State):
 
         """
         my_pid = os.getpid()
-        if name in self.consumers[consumer].processes:
-            child = self.consumers[consumer].processes[name]
-            if child.is_alive():
-                if child.pid != my_pid:
+        if name in self.consumers[consumer].processes.keys():
+            try:
+                child = self.consumers[consumer].processes[name]
+            except KeyError:
+                return
+            try:
+                alive = child.is_alive()
+            except AssertionError:
+                LOGGER.debug('Tried to test non-child process (%r to %r)',
+                             os.getpid(), child.pid)
+            else:
+                if child.pid == my_pid:
+                    LOGGER.debug('Child has my pid? %r, %r', my_pid, child.pid)
+                elif alive:
                     try:
                         child.terminate()
                     except OSError:
                         pass
-                else:
-                    LOGGER.debug('Child has my pid? %r, %r', my_pid, child.pid)
-            del self.consumers[consumer].processes[name]
+            try:
+                del self.consumers[consumer].processes[name]
+            except KeyError:
+                pass
 
     def run(self):
         """When the consumer is ready to start running, kick off all of our
@@ -514,8 +561,8 @@ class MasterControlProgram(state.State):
         self.set_state(self.STATE_ACTIVE)
         self.setup_consumers()
 
-        # Set the SIGABRT handler for child creation errors
-        signal.signal(signal.SIGABRT, self.on_abort)
+        # Set the SIGCHLD handler for child creation errors
+        signal.signal(signal.SIGCHLD, self.on_sigchld)
 
         # Set the SIGALRM handler for poll interval
         signal.signal(signal.SIGALRM, self.on_timer)
@@ -565,7 +612,8 @@ class MasterControlProgram(state.State):
 
         """
         for name in self.consumer_cfg.keys():
-            self.consumers[name] = self.new_consumer(self.consumer_cfg[name])
+            self.consumers[name] = self.new_consumer(self.consumer_cfg[name],
+                                                     name)
             self.start_processes(name, self.consumers[name].qty)
 
     def start_process(self, name):
@@ -581,7 +629,15 @@ class MasterControlProgram(state.State):
         self.consumers[name].processes[process_name] = proc
 
         # Start the process
-        proc.start()
+        try:
+            proc.start()
+        except (OSError, IOError) as error:
+            LOGGER.critical('Failed to start %s for %s: %r',
+                            process_name, name, error)
+            try:
+                del self.consumers[name].process[process_name]
+            except AttributeError as error:
+                LOGGER.warning('Could not cleanup consumer process: %s', error)
 
     def start_processes(self, name, quantity):
         """Start the specified quantity of consumer processes for the given
@@ -591,7 +647,7 @@ class MasterControlProgram(state.State):
         :param int quantity: The quantity of processes to start
 
         """
-        [self.start_process(name) for _i in range(0, quantity)]
+        [self.start_process(name) for _i in range(0, quantity or 0)]
 
     def stop_processes(self):
         """Iterate through all of the consumer processes shutting them down."""
@@ -608,7 +664,10 @@ class MasterControlProgram(state.State):
         LOGGER.info('Sending SIGABRT to active children')
         for proc in multiprocessing.active_children():
             if int(proc.pid) != os.getpid():
-                os.kill(int(proc.pid), signal.SIGABRT)
+                try:
+                    os.kill(int(proc.pid), signal.SIGABRT)
+                except OSError:
+                    pass
 
         # Wait for them to finish up to MAX_SHUTDOWN_WAIT
         for iteration in range(0, self.MAX_SHUTDOWN_WAIT):
