@@ -4,84 +4,59 @@ connection state and collects stats about the consuming process.
 
 """
 import collections
-import importlib
 import logging
 import math
 import multiprocessing
 import os
 from os import path
+import signal
+import time
+import warnings
 try:
     import cProfile as profile
 except ImportError:
     import profile
-import signal
-import sys
-import time
-import warnings
 
 try:
     import sprockets_influxdb as influxdb
 except ImportError:
     influxdb = None
-from tornado import gen, ioloop, locks
 import pika
+from pika import exceptions, spec
+from pika.adapters import tornado_connection
 try:
     import raven
     from raven import breadcrumbs
     from raven.contrib.tornado import AsyncSentryClient
 except ImportError:
     breadcrumbs, raven, AsyncSentryClient = None, None, None
-from pika import spec
+from tornado import gen, ioloop, locks
 
-from rejected import __version__, data, state, statsd, utils
+from . import __version__, data, state, statsd, utils
 
 LOGGER = logging.getLogger(__name__)
 
 Callbacks = collections.namedtuple(
-    'callbacks', ['on_ready', 'on_open_error', 'on_closed', 'on_blocked',
-                  'on_unblocked', 'on_confirmation', 'on_delivery',
-                  'on_return'])
+    'callbacks', ['on_ready', 'on_connection_failure', 'on_closed',
+                  'on_blocked', 'on_unblocked', 'on_confirmation',
+                  'on_delivery', 'on_return'])
 Delivery = collections.namedtuple('Delivery', ['connection', 'message'])
 
 
-def import_consumer(value):
-    """Pass in a string in the format of foo.Bar, foo.bar.Baz, foo.bar.baz.Qux
-    and it will return a handle to the class
-
-    :param str value: The consumer class in module.Consumer format
-    :return: tuple(Class, str)
-
-    """
-    parts = value.split('.')
-    import_name = '.'.join(parts[0:-1])
-    import_handle = importlib.import_module(import_name)
-    version = None
-    if hasattr(import_handle, '__version__'):
-        version = import_handle.__version__
-    elif len(parts) > 2:
-        package_handle = importlib.import_module(parts[0])
-        if hasattr(package_handle, '__version__'):
-            version = package_handle.__version__
-
-    # Return the class handle
-    return getattr(import_handle, parts[-1]), version
-
-
 class Connection(state.State):
 
     HB_INTERVAL = 300
     STATE_CLOSED = 0x08
 
-    def __init__(self, name, config, should_consume, publisher_confirmations,
-                 io_loop, callbacks):
+    def __init__(self, name, config, consumer_name, should_consume,
+                 publisher_confirmations, io_loop, callbacks):
         super(Connection, self).__init__()
         self.blocked = False
         self.callbacks = callbacks
         self.channel = None
         self.config = config
         self.should_consume = should_consume
-        self.consumer_tag = None
-        self.id = 0
+        self.consumer_tag = '{}-{}'.format(consumer_name, os.getpid())
         self.io_loop = io_loop
         self.name = name
         self.publisher_confirm = publisher_confirmations
@@ -96,13 +71,14 @@ class Connection(state.State):
 
     def connect(self):
         self.set_state(self.STATE_CONNECTING)
-        return pika.TornadoConnection(
+        connection = tornado_connection.TornadoConnection(
             self._connection_parameters,
-            on_open_callback=self.on_open,
-            on_open_error_callback=self.on_open_error,
-            on_close_callback=self.on_closed,
             stop_ioloop_on_close=False,
             custom_ioloop=self.io_loop)
+        connection.add_on_close_callback(self.on_closed)
+        connection.add_on_open_callback(self.on_open)
+        connection.add_on_open_error_callback(self.on_open_error)
+        return connection
 
     def shutdown(self):
         if self.is_shutting_down:
@@ -119,28 +95,38 @@ class Connection(state.State):
         else:
             self.channel.close()
 
-    def on_open(self, _handle):
+    def on_open(self, handle):
         """Invoked when the connection is opened
 
-        :type _handle: pika.adapters.tornado_connection.TornadoConnection
+        :type handle: pika.adapters.tornado_connection.TornadoConnection
 
         """
-        LOGGER.debug('Connection %s is open', self.name)
-        self.handle.channel(self.on_channel_open)
+        LOGGER.debug('Connection %s is open (%r)', self.name, handle)
+        self.handle = handle
+        try:
+            self.handle.channel(self.on_channel_open)
+        except exceptions.ConnectionClosed:
+            LOGGER.warning('Channel open on closed connection')
+            self.set_state(self.STATE_CLOSED)
+            self.callbacks.on_closed(self.name)
+            return
         self.handle.add_on_connection_blocked_callback(self.on_blocked)
         self.handle.add_on_connection_unblocked_callback(self.on_unblocked)
 
     def on_open_error(self, *args, **kwargs):
         LOGGER.error('Connection %s failure %r %r', self.name, args, kwargs)
-        self.set_state(self.STATE_CLOSED)
-        self.callbacks.on_open_error(self.name)
+        self.on_failure()
 
     def on_closed(self, _connection, status_code, status_text):
-        self.set_state(self.STATE_CLOSED)
-        LOGGER.debug('Connection %s closed (%s) %s',
-                     self.name, status_code, status_text)
-        self.consumer_tag = None
-        self.callbacks.on_closed(self.name)
+        if self.is_connecting:
+            LOGGER.error('Connection %s failure while connecting (%s): %s',
+                         self.name, status_code, status_text)
+            self.on_failure()
+        elif not self.is_closed:
+            self.set_state(self.STATE_CLOSED)
+            LOGGER.info('Connection %s closed (%s) %s',
+                        self.name, status_code, status_text)
+            self.callbacks.on_closed(self.name)
 
     def on_blocked(self, frame):
         LOGGER.warning('Connection %s is blocked: %r', frame)
@@ -182,16 +168,32 @@ class Connection(state.State):
         :param str reply_text: The AMQP reply text
 
         """
-        LOGGER.debug('Connection %s channel was closed: (%s) %s',
-                     self.name, reply_code, reply_text)
         del self.channel
-        if self.is_running:
-            self.set_state(self.STATE_CONNECTING)
-            self.consumer_tag = None
-            self.handle.channel(self.on_channel_open)
+        if reply_code <= 0 or reply_code == 404:
+            LOGGER.error('Channel Error (%r): %s',
+                         reply_code, reply_text or 'unknown')
+            self.on_failure()
         elif self.is_shutting_down:
             LOGGER.debug('Connection %s closing', self.name)
             self.handle.close()
+        elif self.is_running:
+            LOGGER.warning('Connection %s channel was closed: (%s) %s',
+                           self.name, reply_code, reply_text)
+            try:
+                self.handle.channel(self.on_channel_open)
+            except exceptions.ConnectionClosed as error:
+                LOGGER.warning('Error raised while creating new channel: %s',
+                               error)
+                self.on_failure()
+            else:
+                self.set_state(self.STATE_CONNECTING)
+
+    def on_failure(self):
+        LOGGER.info('Connection failure, terminating connection')
+        self.set_state(self.STATE_CLOSED)
+        self.handle.close()
+        del self.handle
+        self.callbacks.on_connection_failure(self.name)
 
     def consume(self, queue_name, no_ack, prefetch_count):
         self.set_state(self.STATE_ACTIVE)
@@ -199,7 +201,7 @@ class Connection(state.State):
         self.channel.basic_consume(consumer_callback=self.on_delivery,
                                    queue=queue_name,
                                    no_ack=no_ack,
-                                   consumer_tag=self.name)
+                                   consumer_tag=self.consumer_tag)
 
     def on_qos_set(self, frame):
         """Invoked by pika when the QoS is set
@@ -207,7 +209,7 @@ class Connection(state.State):
         :param pika.frame.Frame frame: The QoS Frame
 
         """
-        LOGGER.debug("Connection %s QoS was set: %r", self.name, frame)
+        LOGGER.debug('Connection %s QoS was set: %r', self.name, frame)
 
     def on_consumer_cancelled(self, frame):
         """Invoked by pika when a ``Basic.Cancel`` or ``Basic.CancelOk``
@@ -216,12 +218,10 @@ class Connection(state.State):
         :param pika.frame.Frame frame: The QoS Frame
 
         """
-        LOGGER.debug('Connection %s consumer has been cancelled', self.name)
-        self.consumer_tag = None
-        if self.is_shutting_down:
-            self.channel.close()
-        else:
-            self.set_state(self.STATE_IDLE)
+        LOGGER.info('Connection %s consumer has been cancelled', self.name)
+        if not self.is_shutting_down:
+            self.set_state(self.STATE_SHUTTING_DOWN)
+        self.channel.close()
 
     def on_confirmation(self, frame):
         """Invoked by pika when RabbitMQ responds to a Basic.Publish RPC
@@ -237,7 +237,7 @@ class Connection(state.State):
         LOGGER.debug('Connection %s received delivery confirmation '
                      '(Delivered: %s)', self.name, delivered)
         self.callbacks.on_confirmation(
-            self.name, delivered, frame.delivery_tag)
+            self.name, delivered, frame.method.delivery_tag)
 
     def on_delivery(self, channel, method, properties, body):
         self.callbacks.on_delivery(
@@ -260,6 +260,7 @@ class Connection(state.State):
             pika.PlainCredentials(
                 self.config.get('user', 'guest'),
                 self.config.get('password', self.config.get('pass', 'guest'))),
+            ssl=self.config.get('ssl', False),
             frame_max=self.config.get('frame_max', spec.FRAME_MAX_SIZE),
             socket_timeout=self.config.get('socket_timeout', 10),
             heartbeat_interval=self.config.get(
@@ -289,8 +290,6 @@ class Process(multiprocessing.Process, state.State):
     TIME_SPENT = 'processing_time'
     TIME_WAITED = 'idle_time'
 
-    MESSAGE_AGE = 'message_age'
-
     CONSUMER_EXCEPTION = 'consumer_exception'
     MESSAGE_EXCEPTION = 'message_exception'
     PROCESSING_EXCEPTION = 'processing_exception'
@@ -379,21 +378,27 @@ class Process(multiprocessing.Process, state.State):
         LOGGER.debug('Message processing velocity: %.2f/s', velocity)
         return velocity
 
-    def create_rabbitmq_connections(self):
+    def create_connections(self):
         """Create and start the RabbitMQ connections, assigning the connection
         object to the connections dict.
 
         """
         self.set_state(self.STATE_CONNECTING)
-        for connection in self.consumer_config['connections']:
+        for connection in self.consumer_config.get('connections', []):
             name, confirm, consume = connection, False, True
             if isinstance(connection, dict):
                 name = connection['name']
-                confirm = connection.get('publisher_confirmation', True)
+                confirm = connection.get('publisher_confirmation', False)
                 consume = connection.get('consume', True)
+
+            if name not in self.config['Connections']:
+                LOGGER.critical('Connection "%s" for %s not found',
+                                name, self.consumer_name)
+                continue
+
             self.connections[name] = Connection(
-                name, self.connection_config[name], consume, confirm,
-                self.ioloop, self.callbacks)
+                name, self.config['Connections'][name], self.consumer_name,
+                consume, confirm, self.ioloop, self.callbacks)
 
     @staticmethod
     def get_config(cfg, number, name, connection):
@@ -421,7 +426,7 @@ class Process(multiprocessing.Process, state.State):
 
         """
         try:
-            consumer_, version = import_consumer(cfg['consumer'])
+            handle, version = utils.import_consumer(cfg['consumer'])
         except ImportError as error:
             LOGGER.exception('Error importing the consumer %s: %s',
                              cfg['consumer'], error)
@@ -433,12 +438,13 @@ class Process(multiprocessing.Process, state.State):
         else:
             LOGGER.info('Creating consumer %s', cfg['consumer'])
 
-        settings = cfg.get('config', dict())
+        settings = cfg.get('config', {})
         settings['_import_module'] = '.'.join(cfg['consumer'].split('.')[0:-1])
 
         kwargs = {
             'settings': settings,
             'process': self,
+            'drop_exchange': cfg.get('drop_exchange'),
             'drop_invalid_messages': cfg.get('drop_invalid_messages'),
             'message_type': cfg.get('message_type'),
             'error_exchange': cfg.get('error_exchange'),
@@ -446,12 +452,12 @@ class Process(multiprocessing.Process, state.State):
         }
 
         try:
-            return consumer_(**kwargs)
+            return handle(**kwargs)
         except Exception as error:
             LOGGER.exception('Error creating the consumer "%s": %s',
                              cfg['consumer'], error)
 
-    @gen.engine
+    @gen.coroutine
     def invoke_consumer(self, message):
         """Wrap the actual processor processing bits
 
@@ -471,13 +477,6 @@ class Process(multiprocessing.Process, state.State):
                     self.counters[self.REDELIVERED] += 1
                     self.measurement.set_tag(self.REDELIVERED, True)
 
-                if message.properties.timestamp:
-                    self.measurement.set_value(
-                        self.MESSAGE_AGE,
-                        float(
-                            max(message.properties.timestamp, start_time) -
-                            message.properties.timestamp))
-
                 try:
                     result = yield self.consumer.execute(message,
                                                          self.measurement)
@@ -492,11 +491,11 @@ class Process(multiprocessing.Process, state.State):
             elif self.is_waiting_to_shutdown:
                 LOGGER.info(
                     'Requeueing pending message due to pending shutdown')
-                self.reject(message.delivery_tag, True)
+                self.reject(message, True)
                 self.shutdown_connections()
             elif self.is_shutting_down:
                 LOGGER.info('Requeueing pending message due to shutdown')
-                self.reject(message.delivery_tag, True)
+                self.reject(message, True)
                 self.on_ready_to_stop()
             else:
                 LOGGER.warning('Exiting invoke_consumer without processing, '
@@ -527,41 +526,50 @@ class Process(multiprocessing.Process, state.State):
 
     def on_connection_closed(self, name):
         if self.is_running:
+            LOGGER.warning('Connection %s was closed, reconnecting', name)
             return self.connections[name].connect()
 
-        ready = all([c.is_closed for c in self.connections.values()])
+        ready = all(c.is_closed for c in self.connections.values())
         if (self.is_shutting_down or self.is_waiting_to_shutdown) and ready:
             self.on_ready_to_stop()
 
-    def on_connection_failure(self, name):
-        LOGGER.debug('Connection %s indicated it failed to connect', name)
-        ready = all([c.is_closed for c in self.connections.values()])
-        if (self.is_connecting or
-                self.is_shutting_down or
+    def on_connection_failure(self, *args, **kwargs):
+        ready = all(c.is_closed for c in self.connections.values())
+        LOGGER.warning('Connection failure while %s - Ready to stop: %r',
+                       self.state_description, ready)
+        if (self.is_connecting or self.is_idle or self.is_shutting_down or
                 self.is_waiting_to_shutdown) and ready:
             self.on_ready_to_stop()
 
     def on_connection_ready(self, name):
         LOGGER.debug('Connection %s indicated it is ready', name)
         self.consumer.set_channel(name, self.connections[name].channel)
-        if self.connections[name].should_consume:
-            self.connections[name].consume(
-                self.consumer_config['queue'], self.no_ack,
-                self.consumer_config['qos_prefetch'])
+        if all(c.is_idle for c in self.connections.values()):
+            for key in self.connections.keys():
+                if self.connections[key].should_consume:
+                    self.connections[key].consume(
+                        self.queue_name, self.no_ack, self.qos_prefetch)
             if self.is_connecting:
                 self.set_state(self.STATE_IDLE)
 
     def on_connection_blocked(self, name):
-        LOGGER.debug('Connection %s blocked', name)
+        LOGGER.warning('Connection %s blocked', name)
         if self.is_processing:
             self.consumer.on_blocked(name)
 
     def on_connection_unblocked(self, name):
-        LOGGER.debug('Connection %s unblocked', name)
+        LOGGER.info('Connection %s unblocked', name)
         if self.is_processing:
             self.consumer.on_blocked(name)
 
     def on_confirmation(self, name, delivered, delivery_tag):
+        """Invoked on delivery confirmation
+
+        :param str name: The RabbitMQ connection that confirmed the delivery
+        :param bool delivered: Was the message was successfully delivered
+        :param str delivery_tag: The delivery tag for the message
+
+        """
         if self.is_processing:
             self.consumer.on_confirmation(name, delivered, delivery_tag)
 
@@ -569,7 +577,7 @@ class Process(multiprocessing.Process, state.State):
         """Process a message from Rabbit
 
         :param str name: The connection name
-        :param pika.channel.Channel channel: The channel the message was sent on
+        :param pika.channel.Channel channel: The message's delivery channel
         :param pika.frames.MethodFrame method: The method frame
         :param pika.spec.BasicProperties properties: The message properties
         :param str body: The message body
@@ -610,7 +618,7 @@ class Process(multiprocessing.Process, state.State):
         """
         duration = max(start_time, time.time()) - start_time
         self.counters[self.TIME_SPENT] += duration
-        self.measurement.set_value(self.TIME_SPENT, duration)
+        self.measurement.add_duration(self.TIME_SPENT, duration)
 
         if result == data.MESSAGE_DROP:
             LOGGER.debug('Rejecting message due to drop return from consumer')
@@ -621,27 +629,26 @@ class Process(multiprocessing.Process, state.State):
             LOGGER.debug('Rejecting message due to MessageException')
             self.reject(message, False)
             self.counters[self.MESSAGE_EXCEPTION] += 1
-            self.measurement.set_tag('exception', 'MessageException')
 
         elif result == data.PROCESSING_EXCEPTION:
             LOGGER.debug('Rejecting message due to ProcessingException')
-            self.reject(message, False)
+            if self.consumer.ACK_PROCESSING_EXCEPTIONS:
+                self.ack_message(message)
+            else:
+                self.reject(message, False)
             self.counters[self.PROCESSING_EXCEPTION] += 1
-            self.measurement.set_tag('exception', 'ProcessingException')
 
         elif result == data.CONSUMER_EXCEPTION:
             LOGGER.debug('Re-queueing message due to ConsumerException')
             self.reject(message, True)
             self.on_processing_error()
             self.counters[self.CONSUMER_EXCEPTION] += 1
-            self.measurement.set_tag('exception', 'ConsumerException')
 
         elif result == data.UNHANDLED_EXCEPTION:
             LOGGER.debug('Re-queueing message due to UnhandledException')
             self.reject(message, True)
             self.on_processing_error()
             self.counters[self.UNHANDLED_EXCEPTION] += 1
-            self.measurement.set_tag('exception', 'UnhandledException')
 
         elif result == data.MESSAGE_REQUEUE:
             LOGGER.debug('Re-queueing message due Consumer request')
@@ -652,6 +659,7 @@ class Process(multiprocessing.Process, state.State):
             self.ack_message(message)
 
         self.counters[self.PROCESSED] += 1
+        self.measurement.set_tag(self.PROCESSED, True)
         self.maybe_submit_measurement()
         self.reset_state()
 
@@ -673,6 +681,8 @@ class Process(multiprocessing.Process, state.State):
             self.shutdown_connections()
 
     def on_ready_to_stop(self):
+        """Invoked when the consumer is ready to stop."""
+        LOGGER.debug('Ready to stop')
 
         # Set the state to shutting down if it wasn't set as that during loop
         self.set_state(self.STATE_SHUTTING_DOWN)
@@ -684,7 +694,8 @@ class Process(multiprocessing.Process, state.State):
         signal.signal(signal.SIGTERM, signal.SIG_IGN)
 
         # Allow the consumer to gracefully stop and then stop the IOLoop
-        self.stop_consumer()
+        if self.consumer:
+            self.stop_consumer()
 
         # Clear IOLoop constructs
         self.consumer_lock = None
@@ -710,6 +721,14 @@ class Process(multiprocessing.Process, state.State):
         self.last_stats_time = time.time()
         signal.siginterrupt(signal.SIGPROF, False)
 
+    def on_startup_error(self, error):
+        """Invoked when a pre-condition for starting the consumer has failed.
+        Log the error and then exit the process.
+
+        """
+        LOGGER.critical('Could not start %s: %s', self.consumer_name, error)
+        self.set_state(self.STATE_STOPPED)
+
     def reject(self, message, requeue=True):
         """Reject the message on the broker and log it.
 
@@ -736,7 +755,7 @@ class Process(multiprocessing.Process, state.State):
     def report_stats(self):
         """Create the dict of stats data for the MCP stats queue"""
         if not self.previous:
-            self.previous = dict()
+            self.previous = {}
             for key in self.counters:
                 self.previous[key] = 0
         values = {
@@ -790,17 +809,15 @@ class Process(multiprocessing.Process, state.State):
         self.consumer_lock = locks.Lock()
 
         self.sentry_client = self.setup_sentry(
-            self._kwargs['config'], self._kwargs['consumer_name'])
+            self._kwargs['config'], self.consumer_name)
 
         try:
             self.setup()
         except (AttributeError, ImportError) as error:
-            name = self._kwargs['consumer_name']
-            class_name = self._kwargs['config']['Consumers'][name]['consumer']
-            LOGGER.exception('Could not start %s, stopping process: %r',
-                             class_name, error)
-            os.kill(os.getppid(), signal.SIGABRT)
-            sys.exit(1)
+            LOGGER.exception('Setup failure: %s', error)
+            return self.on_startup_error(
+                'Failed to import the Python module for {}'.format(
+                    self.consumer_name))
 
         if not self.is_stopped:
             try:
@@ -824,11 +841,12 @@ class Process(multiprocessing.Process, state.State):
             duration = math.ceil(time.time() - self.delivery_time) * 1000
         except TypeError:
             duration = 0
-        kwargs = {'extra': {
-                      'consumer_name': self.consumer_name,
-                      'env': dict(os.environ),
-                      'message': message},
-                  'time_spent': duration}
+        kwargs = {
+            'extra': {
+                'consumer_name': self.consumer_name,
+                'env': dict(os.environ),
+                'message': message},
+            'time_spent': duration}
         LOGGER.debug('Sending exception to sentry: %r', kwargs)
         self.sentry_client.captureException(exc_info, **kwargs)
 
@@ -838,18 +856,23 @@ class Process(multiprocessing.Process, state.State):
 
         """
         LOGGER.info('Initializing for %s', self.name)
+
+        if 'consumer' not in self.consumer_config:
+            return self.on_startup_error(
+                '"consumer" not specified in configuration')
+
         self.consumer = self.get_consumer(self.consumer_config)
 
         if not self.consumer:
-            LOGGER.critical('Could not import and start processor')
-            self.set_state(self.STATE_STOPPED)
-            self.on_ready_to_stop()
-            return
+            return self.on_startup_error(
+                'Could not import "{}"'.format(
+                    self.consumer_config.get(
+                        'consumer', 'unconfigured consumer')))
 
         self.setup_instrumentation()
         self.reset_error_counter()
         self.setup_sighandlers()
-        self.create_rabbitmq_connections()
+        self.create_connections()
 
     def setup_influxdb(self, config):
         """Configure the InfluxDB module for measurement submission.
@@ -888,6 +911,9 @@ class Process(multiprocessing.Process, state.State):
         if not self.config.get('stats') and not self.config.get('statsd'):
             return
 
+        if 'stats' not in self.config:
+            self.config['stats'] = {}
+
         # Backwards compatible statsd config support
         if self.config.get('statsd'):
             warnings.warn('Deprecated statsd configuration detected',
@@ -898,7 +924,8 @@ class Process(multiprocessing.Process, state.State):
         if self.config['stats'].get('statsd'):
             if self.config['stats']['statsd'].get('enabled', True):
                 self.statsd = statsd.Client(self.consumer_name,
-                                            self.config['stats']['statsd'])
+                                            self.config['stats']['statsd'],
+                                            self.stop)
             LOGGER.debug('statsd measurements configured')
 
         # InfluxDB support
@@ -1009,6 +1036,20 @@ class Process(multiprocessing.Process, state.State):
             measurement.set_tag(key, value)
         for key, value in self.measurement.values.items():
             measurement.set_field(key, value)
+
+        for key, values in self.measurement.durations.items():
+            if len(values) == 1:
+                measurement.set_field(key, values[0])
+            elif len(values) > 1:
+                measurement.set_field('{}-average'.format(key),
+                                      sum(values) / len(values))
+                measurement.set_field('{}-max'.format(key), max(values))
+                measurement.set_field('{}-min'.format(key), min(values))
+                measurement.set_field('{}-median'.format(key),
+                                      utils.percentile(values, 50))
+                measurement.set_field('{}-95th'.format(key),
+                                      utils.percentile(values, 95))
+
         influxdb.add_measurement(measurement)
         LOGGER.debug('InfluxDB Measurement: %r', measurement.marshall())
 
@@ -1016,11 +1057,11 @@ class Process(multiprocessing.Process, state.State):
         """Submit a measurement for a message to statsd as individual items."""
         for key, value in self.measurement.counters.items():
             self.statsd.incr(key, value)
-        for key, value in self.measurement.values.items():
-            if isinstance(value, float):
+        for key, values in self.measurement.durations.items():
+            for value in values:
                 self.statsd.add_timing(key, value)
-            else:
-                self.statsd.set_gauge(key, value)
+        for key, value in self.measurement.values.items():
+            self.statsd.set_gauge(key, value)
         for key, value in self.measurement.tags.items():
             if isinstance(value, bool):
                 if value:
@@ -1043,13 +1084,9 @@ class Process(multiprocessing.Process, state.State):
     def config(self):
         return self._kwargs['config']
 
-    @property
-    def connection_config(self):
-        return self.config['Connections']
-
     @property
     def consumer_config(self):
-        return self.config['Consumers'][self.consumer_name]
+        return self.config['Consumers'][self.consumer_name] or {}
 
     @property
     def consumer_name(self):
